--[[
ModuleName : START_SETTING_WINDOW
Path : base/startsettingwindow.lua
Author : luyafei
CreateTime : 2019-03-02 16:30:02
Description : description
--]]

---------------------------------------------------------------------------------------------
local UnityEngine_Application_persistentDataPath = UnityEngine.Application.persistentDataPath
local UnityEngine_Object_Destroy = UnityEngine.Object.Destroy
local UnityEngine_Object_Instantiate = UnityEngine.Object.Instantiate
local UnityEngine_GameObject = UnityEngine.GameObject
local UnityEngine_UI_Button = UnityEngine.UI.Button
local UnityEngine_UI_Dropdown = UnityEngine.UI.Dropdown
local UnityEngine_Resources_Load = UnityEngine.Resources.Load
local UnityEngine_UI_Text = UnityEngine.UI.Text
---------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------
local downloaderrmsg = "download failed"
local windowAssetPath = "startsettingres/ui_start_setting_window"
local requestUrl = "http://192.168.1.3:50160/start.json"
local window = false
local infoText = false
local operatorDropdown = false
local gameworldDropdown = false
local jsonData = false
local gotoCallback = false
local downloadRetStr = false
local debugSaveToFile = "savePath.txt"

local ACTION_DOWNLOAD = "download"
local ACTION_CLEARCACHE = "clearcache"
---------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------
local function SendGet(url, onSuccessCallback, onFailCallback)
	N1HttpLuaTool.DoRequest(url, onSuccessCallback, onFailCallback)
end

local function DoDownLoad(url, saveto)
	SendGet(url, function(retStr)
		local fullPath = UnityEngine_Application_persistentDataPath .. "/" .. saveto
		local fLua, msg = io.open(fullPath, "w+")
		if fLua then
			fLua:write(retStr)
			fLua:close()

			-- 这边把saveto保存下来
			local savePath = UnityEngine_Application_persistentDataPath .. "/" .. debugSaveToFile
			local file = io.open(savePath, "a+")
			if file then
				file:write(saveto)
				file:write("\n")
				file:close()
			end
		end
		infoText.text = retStr
	end, function()
		infoText.text = downloaderrmsg
	end)
end

local function DoClearCache()
	local savePath = UnityEngine_Application_persistentDataPath .. "/" .. debugSaveToFile
	local file = io.open(savePath, "r")
	if file then
		file:close()
	end
end

local function AnalysisJsonData(retStr)
	return cjson.decode(retStr)
end

local function SetUIData()
	if not jsonData then
		return
	end

	local operators = jsonData.operators

	local operatorList = ListString()
	for idx, gwinfo in ipairs(operators) do
		local name = gwinfo.name
		operatorList:Add(name)
	end
	operatorDropdown:AddOptions(operatorList)
end

local function closeWindow()
	if not window then
		return
	end
	UnityEngine_Object_Destroy(window)
end

local function OnUICloseCallback()
	closeWindow()

	if gotoCallback then
		gotoCallback()
	end
end

local function OnUIConfirmCallback()
	local operatorVal = operatorDropdown.value or 0
	local gamewroldVal = gameworldDropdown.value or 0

	local operators = jsonData.operators
	local gwinfo = operators[operatorVal + 1]
	local gameworlds = gwinfo.gameworlds
	local actioninfo = gameworlds[gamewroldVal + 1]
	local actions = actioninfo.actions
	for idx, actioninfo in ipairs(actions) do
		local action = actioninfo.action
		if action == ACTION_DOWNLOAD then
			local url = actioninfo.url
			local saveto = actioninfo.saveto
			DoDownLoad(url, saveto)
		end
		if action == ACTION_CLEARCACHE then
			DoClearCache()
		end
	end
end

local function OnUIOperatorCallback(idx)
	local val = operatorDropdown.value or 0

	local operators = jsonData.operators
	local gwinfo = operators[val + 1]
	local gameworlds = gwinfo.gameworlds

	local gameworldsList = ListString()
	for idx, actions in ipairs(gameworlds) do
		gameworldsList:Add(actions.name)
	end
	gameworldDropdown:ClearOptions()
	gameworldDropdown:AddOptions(gameworldsList)
end

local function onSuccessCallback(retStr)
	local windowAsset = UnityEngine_Resources_Load(windowAssetPath)
	window = UnityEngine_Object_Instantiate(windowAsset)

	local closeGo = window.transform:Find("panel/close").gameObject
	local closeBtnCom = closeGo:GetComponent(UnityEngine_UI_Button)
	closeBtnCom.onClick:AddListener(OnUICloseCallback)

    local confirmGo = window.transform:Find("panel/confirm").gameObject
    local confirmBtnCom = confirmGo:GetComponent(UnityEngine_UI_Button)
    confirmBtnCom.onClick:AddListener(OnUIConfirmCallback)

    local operatorGo = window.transform:Find("panel/operator").gameObject
    operatorDropdown = operatorGo:GetComponent(UnityEngine_UI_Dropdown)
    operatorDropdown:ClearOptions()
    operatorDropdown.onValueChanged:AddListener(OnUIOperatorCallback)

    local gameworldGo = window.transform:Find("panel/gameworld").gameObject
    gameworldDropdown = gameworldGo:GetComponent(UnityEngine_UI_Dropdown)
    gameworldDropdown:ClearOptions()

    local infoGo = window.transform:Find("panel/info").gameObject
    infoText = infoGo:GetComponent(UnityEngine_UI_Text)

    jsonData = AnalysisJsonData(retStr)
    SetUIData()
    OnUIOperatorCallback()
end

local function onFailCallback(msg)
	closeWindow()

	if gotoCallback then
		gotoCallback()
	end
end

return function (callback)
	gotoCallback = callback
	SendGet(requestUrl, onSuccessCallback, onFailCallback)
end
---------------------------------------------------------------------------------------------
